# Generated by Django 3.2.7 on 2021-10-02 20:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("denorm", "0003_auto_20211002_1955"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dirtyinstance",
            name="success",
            field=models.NullBooleanField(),
        ),
    ]
