# Generated by Django 4.2.3 on 2023-07-09 20:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("denorm", "0001_squashed_0012_alter_dirtyinstance_object_id"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dirtyinstance",
            name="success",
            field=models.BooleanField(db_index=True, null=True),
        ),
    ]
