# Generated by Django 3.2.7 on 2021-10-03 03:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("denorm", "0006_auto_20211003_0346"),
    ]

    operations = [
        migrations.RunSQL(
            "ALTER TABLE denorm_dirtyinstance ALTER COLUMN created_on SET DEFAULT CURRENT_TIMESTAMP",
            migrations.RunSQL.noop,
        )
    ]
