# Generated by Django 4.2.23 on 2025-08-18 09:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("denorm", "0014_parametrized_notify"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="dirtyinstance",
            name="processing_finished",
        ),
        migrations.RemoveField(
            model_name="dirtyinstance",
            name="processing_started",
        ),
        migrations.RemoveField(
            model_name="dirtyinstance",
            name="success",
        ),
        migrations.RemoveField(
            model_name="dirtyinstance",
            name="traceback",
        ),
    ]
