# Generated by Django 4.2.3 on 2023-07-09 20:28

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("test_app", "0007_alter_post_response_count"),
    ]

    operations = [
        migrations.AlterField(
            model_name="attachment",
            name="forum",
            field=models.ForeignKey(
                blank=True,
                default=None,
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="test_app.forum",
            ),
        ),
    ]
