# Generated by Django 3.2.7 on 2021-10-02 17:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("denorm", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="dirtyinstance",
            name="func_name",
            field=models.TextField(blank=True, null=True),
        ),
    ]
