# Generated by Django 4.2.3 on 2023-07-09 20:23

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("test_app", "0005_skipcommentwithonly"),
    ]

    operations = [
        migrations.AlterField(
            model_name="forum",
            name="author_names",
            field=models.CharField(default="", editable=False, max_length=255),
        ),
    ]
