# Generated by Django 3.2.7 on 2021-10-02 01:13

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("test_app", "0004_auto_20160306_1822"),
    ]

    operations = [
        migrations.CreateModel(
            name="SkipCommentWithOnly",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.TextField()),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("updated_on", models.DateTimeField(auto_now=True, null=True)),
                ("post_text", models.TextField(editable=False)),
                (
                    "post",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="test_app.skippost",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
