# Generated by Django 3.2.7 on 2021-10-03 03:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("denorm", "0005_dirtyinstance_created_on"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dirtyinstance",
            name="created_on",
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="dirtyinstance",
            name="func_name",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name="dirtyinstance",
            name="object_id",
            field=models.IntegerField(db_index=True, default=0),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="dirtyinstance",
            name="processing_started",
            field=models.DateTimeField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name="dirtyinstance",
            name="success",
            field=models.NullBooleanField(db_index=True),
        ),
    ]
