# Generated by Django 4.2.3 on 2023-07-09 20:29

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("test_app", "0009_alter_callcounter_called_count"),
    ]

    operations = [
        migrations.AlterField(
            model_name="callcounter",
            name="called_count",
            field=models.IntegerField(default=None, editable=False),
        ),
    ]
