# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-03-06 18:22
from __future__ import unicode_literals

import denorm.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0003_auto_20160218_1341'),
    ]

    operations = [
        migrations.CreateModel(
            name='Competitor',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='Team',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user_string', models.TextField(editable=False)),
            ],
        ),
        migrations.AlterField(
            model_name='filtercountitem',
            name='text',
            field=models.CharField(default='', max_length=10),
        ),
        migrations.AlterField(
            model_name='filtercountmodel',
            name='active_item_count',
            field=denorm.fields.CountField('items', default=0),
        ),
        migrations.AlterField(
            model_name='filtersummodel',
            name='active_item_sum',
            field=denorm.fields.SumField('counts', default=0),
        ),
        migrations.AddField(
            model_name='competitor',
            name='team',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='test_app.Team'),
        ),
    ]
